%Hpdmap
% Probability density map of a Hydrogenic atom. This function plots a slice
% of a cylinder surrounding the atom.
%
% Energy levels in Hydrogenic type atoms are characterized by quantum
% numbers n = 1, 2, 3, 4....
%
% Orbital quantum numbers are:
% L = 0,1,2....(n-1)
% M = 0, +/-1, +/-2,...+/- L
%
% Orbitals are typically labelled in the form nXM e.g. 3P1
% X is characterized by L.
% L = 0, X = S   min(n) = 1   max(abs(M)) = 0
% L = 1, X = P   min(n) = 2   max(abs(M)) = 1
% L = 2, X = D   min(n) = 3   max(abs(M)) = 2
% L = 3, X = F   min(n) = 4   max(abs(M)) = 3
% L = 4, X = G   min(n) = 5   max(abs(M)) = 4
% L = 5, X = H   min(n) = 6   max(abs(M)) = 5
% L = 6, X = I   min(n) = 7   max(abs(M)) = 6
%
% By convention XM orbitals are the linear combination of +abs(M) and
% -abs(M) orbitals. For example 3P-1 will combine the (angular) orbitals
% Y(L=1,M=1) - Y(L=1,M=-1). Similarly 3P1 will correspond to Y(L=1,M=1) +
% Y(L=1,M=-1).
%
% LAST UPDATED by Andy French Dec 2011

function Hdpmap( n, orbital, L, M, Z, A, N, Nz, Wmin, num_z_slices )

%% Compute coordinates in cylindrical polars %%

%Determine mean radius and use this to construct values at which the
%wavefunction is evaluated
[w,E,r_mean,a0] = Hwavfunc( 0, 0, 0, Z, A, n, L, 0 );
rmax = ceil(0.9*r_mean);
zmax = ceil(0.9*r_mean);

%Define cylinder radial distance in Angstroms
r = linspace(0,rmax,N);

%Define azimumth extent /radians
azi = linspace(0,2*pi,2*N);

%Define z extent /Angstroms
z = linspace(-zmax,zmax,Nz);

%Construct matrix of cylinder range and azimuth values
[r,azi] = meshgrid(r,azi);

%Construct x,y matrix
x = r.*sin(azi);
y = r.*cos(azi);

%

%% Compute wavefunction %%

%Determine magnetic quantum numbers 
if isempty(M)
    M = -L:L ;
end

%Step through the 2*N values of z, and the required magnetic quantum numbers,
%and compute slices of the wavefunction probability density. These shall be
%stored in an indexed structure W(nz,m) where nz is the slice number and m
%is the magnetic quantum number index.

%Initialise maximum probability density
wmax = 0;

%Step through magnetic quantum numbers
for m=1:length(M)
    
    disp(['... computing wavefunctions : ','M=',...
        num2str(M(m))])
    
    %Step through z slices
    for nz=1:length(z)
        
        %Construct a matrix of z values
        zz = z(nz)*ones(size(r));
        
        %Compute matrix of elevation /radians
        elev = atan2(zz,r);
        
        %Construct matrix of ranges in Angstroms from the origin
        R = sqrt( r.^2 + zz.^2 );
        
        %Compute wavefunction and therefore probability density
        [w,E,r_mean,a0] = Hwavfunc( R, azi, elev, Z, A, n, L, M(m) );
        w = w.*conj(w);
        
        %Store wavefunction probability density
        W(nz,m).pd = w;
        W(nz,m).z = zz;
        
        %Modify overall maximum probability desnity
        w_max = max(max(w));
        if w_max>wmax
            wmax = w_max;
        end
    end
end

%Normalize wavefunctions by overall maximum, and set = NaN below defined
%cutoff
for m=1:length(M)
    for nz=1:length(z)
        W(nz,m).pd = W(nz,m).pd/wmax;
        W(nz,m).pd( W(nz,m).pd < Wmin ) = NaN;
    end
end

%

%% Plot probability density maps %%

disp('... plotting wavefunctions in 3D')

%Probability density surface vs x,y for given z
figure('color',[1 1 1],'renderer','opengl');
axes;
dim = size(W);
for m=1:dim(2)
    for nz=1:dim(1)
        surf(x,y,W(nz,m).z,W(nz,m).pd);
        alpha(0.2);
        hold on
    end
end
xlim([-rmax,rmax]);
ylim([-rmax,rmax]);
axis tight
axis vis3d
title_str = ['Hydrogenic atom: Z=',num2str(Z),...
    ', A=',num2str(A),', orbital ',num2str(n),orbital,', E=',...
    num2str(E),'eV , M=',num2str(M)];
title(title_str)
xlabel('x in Angstroms')
ylabel('y in Angstroms');
zlabel('z in Angstroms');
shading interp;
% cmap = [1 1 1; 0 0 1; 0 1 0; 1 0 0];
% colormap(cmap);
interp_colormap(500);
caxis([Wmin,1]);
colorbar
saveas(gcf,['H orbitals\z stack 3D H_orbital',' ',orbital],'png')
close(gcf);

%

disp('... plotting wavefunctions in z slices')

%

%Probability density surface vs x,y for given z
figure('color',[1 1 1],'renderer','opengl');
dim = size(W);
z_indices = round(linspace(1,dim(1),num_z_slices));
for nz=1:num_z_slices
    for m=1:dim(2)
        surf(x,y,zeros(size(x)),W(z_indices(nz),m).pd);
        hold on;
    end    
    view(2);
    axis tight
    axis equal
    title_str = ['z=',num2str(z(z_indices(nz))),' angstroms'];
    title(title_str)
    xlabel('x in Angstroms')
    ylabel('y in Angstroms');
    shading interp;
%     cmap = [1 1 1; 0 0 1; 0 1 0; 1 0 0];
%     colormap(cmap);
    interp_colormap(500);
    caxis([Wmin,1]);   
colorbar
    saveas(gcf,['H orbitals\z slice H_orbital',' ',orbital, ' ',leadingzero(nz,2)],'png')
    clf
end
close(gcf);

%Montage sub-images
delete_files = 1;
montage('H orbitals',['z slice H_orbital',' ',orbital],delete_files)

%End of code
