%Hpdmap2
% Probability density map of a Hydrogenic atom.
%
% Energy levels in Hydrogenic type atoms are characterized by quantum
% numbers n = 1, 2, 3, 4....
%
% Orbital quantum numbers are:
% L = 0,1,2....(n-1)
% M = 0, +/-1, +/-2,...+/- L
%
% Orbitals are typically labelled in the form nXM e.g. 3P1
% X is characterized by L.
% L = 0, X = S   min(n) = 1   max(abs(M)) = 0
% L = 1, X = P   min(n) = 2   max(abs(M)) = 1
% L = 2, X = D   min(n) = 3   max(abs(M)) = 2
% L = 3, X = F   min(n) = 4   max(abs(M)) = 3
% L = 4, X = G   min(n) = 5   max(abs(M)) = 4
% L = 5, X = H   min(n) = 6   max(abs(M)) = 5
% L = 6, X = I   min(n) = 7   max(abs(M)) = 6
%
% By convention XM orbitals are the linear combination of +abs(M) and
% -abs(M) orbitals. For example 3P-1 will combine the (angular) orbitals
% Y(L=1,M=1) - Y(L=1,M=-1). Similarly 3P1 will correspond to Y(L=1,M=1) +
% Y(L=1,M=-1).
%
% LAST UPDATED by Andy French Dec 2011

function [x,y,z,W,E,M] = Hdpmap2( n, orbital, L, M, Z, A, N )

%% Compute coordinates %%

%Determine mean radius and use this to construct values at which the
%wavefunction is evaluated
[w,E,r_mean,a0] = Hwavfunc( 0, 0, 0, Z, A, n, L, 0 );
a = ceil(0.9*r_mean);

%Construct x,y,z matrices
x = linspace(-a,a,N);
y = linspace(-a,a,N);
z = linspace(-a,a,N);
[x,y,z]=meshgrid(x,y,z);

%Convert to spherical polars
R = sqrt( x.^2 + y.^2 + z.^2 );
elev = asin( z./R );
azi = atan2( x,y );

%

%% Compute wavefunction %%

%Determine magnetic quantum numbers
if isempty(M)
    M = -L:L ;
end

%Step through the 2*N values of z, and the required magnetic quantum numbers,
%and compute slices of the wavefunction probability density. These shall be
%stored in an indexed structure W(nz,m) where nz is the slice number and m
%is the magnetic quantum number index.

%Initialise maximum probability density
wmax = 0;

%Step through magnetic quantum numbers
for m=1:length(M)
    
    disp(['... computing wavefunctions : ','M=',...
        num2str(M(m))])
    
    %Compute wavefunction and therefore probability density
    [w,E,r_mean,a0] = Hwavfunc( R, azi, elev, Z, A, n, L, M(m) );
    w = w.*conj(w);
    
    %Store wavefunction probability density
    W(m).pd = w;
    
    %Modify overall maximum probability desnity
    w_max = max(max(max((w))));
    if w_max>wmax
        wmax = w_max;
    end
end

%Normalize wavefunctions by overall maximum
for m=1:length(M)
    W(m).pd = W(m).pd/wmax;
end

%End of code
