%Hwavefunc
% Computes values of a 'Hydrogenic' atom wavefunction given matrix inputs of
% range (in Angstroms), azimuth (radians) and elevation (also radians).
%
% LAST UPDATED by Andy French Dec 2011
%
% [w,E,r_mean,a0] = Hwavfunc( r, azi, elev, Z, A, n, L, M )
%
% r, azi, elev      Range, azimuth and elevation matrices. Must be of the
%                   same size.
% Z                 Atomic number
% A                 Atomic mass number
% n                 Principal quantum number
% L                 Orbital quantum number = 0,1,2....(n-1)
% M                 Magnetic quantum number = 0, +/-1, +/-2,...+/- L
%
% If invalid L and M values are supplied, the output w will be NaN.
%
% w                 The wavefunction (solution to Schrodinger's equation)
%                   Note w.*conj(w) is the probability density of finding
%                   an electron.
% E                 Orbital energy in electronvolts. Energy levels in
%                   Hydrogenic type atoms are characterized by quantum
%                   numbers n = 1, 2, 3, 4....
% r_mean            Hydrogenic atom mean radius in Angstroms
% a0                Bohr radius in Angstroms.

function [w,E,r_mean,a0] = Hwavfunc( r, azi, elev, Z, A, n, L, M )

%Check for invalid (NaN) orbital quantum numbers
if isnan(L)==1
    w = NaN(size(r));
    E = NaN;
    a0 = NaN;
    r_mean = NaN;
else
    
    %Initialize wavefunction
    w=zeros(size(r));
    
    %Combine wavefunctions over range of magnetic quantum number M
    for m=1:length(M)
        
        %Radial wavefunction
        [w1,E,r_mean,r2_mean,a0,a] = Hradial(r,n,L,Z,A);
        
        %Angular wavefunction
        if M==0
            w2 = spharm( azi, elev, L, 0 );
        elseif M<0
            M=abs(M);
            w2 = spharm( azi, elev, L, M(m) ) - spharm( azi, elev, L, -M(m) );
        else
            M=abs(M);
            w2 = spharm( azi, elev, L, M(m) ) + spharm( azi, elev, L, -M(m) );
        end
        
        %Multiply angular and radial parts to yield the wavefunction
        %and add this to the previously calculated wavefunction for the
        %previous magnetic quantum number
        w = w1.*w2 + w;
    end
    
    %Scale by number of magnetic quantum numbers
    w = w/length(M);
end

%End of code