%fig2pic
% Function which constructs a bitmap image of a current figure with a
% defined number of dots per inch and aspect ratio. For images larger than
% those that would natually be obtained using the saveas command, a
% sequence of PNG bitmaps are firstly formed from a zoom-in of the figure.
% These are then reassembled to form the output bitmap. For images smaller
% than the 'saveas bitpmap', linear interpolation is used. Blank space will
% be used when the figure aspect ratio is different to width : height.
%
% LAST UPDATED by Andy French 4-July-2011
%
% Syntax: fig2pic( fig, name , bitmap, aspect , DPI , width_cm, zoom_method )
%
% fig is the handle of the figure to be converted into a bitmap
% name is the filename of the output bitmap (no extension)
% bitmap is the extension of the output file
% aspect is the width / height ratio of the output image
% DPI is the number of dots per inch of the output image
% width_cm is the width of the final image in cm
% zoom_method is the method of enlargement. 'interp' interpolates the image
% (which can be slow and won't add any extra detail), otherwise the axes is
% enlarged and shifted to create a montage of figure images, which are then
% stitched together.

function fig2pic( fig, name , bitmap, aspect , DPI , width_cm, zoom_method )

%Start clock
tic;

%Number of centimeters per inch
cm_per_inch = 1/0.3937;

%Pixels per centimeter at desired level of DPI
pixels_per_cm = DPI / cm_per_inch ;

%Determine number of pixels of width and height of final image
width = round( width_cm * pixels_per_cm ) ;
height = round( width / aspect );

%Determine blank colour from figure axes [R,G,B] color property. Values of blank
%are between 0 and 255.
figure(fig);
blank=round(255*get(fig,'color'));

%Make image of current figure and get pixel dimensions
F = getframe(fig); I0=F.cdata;
[height0,width0,num_cols]=size(I0);

%Determine aspect ratio of 'initial_image'
aspect0 = width0/height0;

%Depending on aspect ratios of initial_image and desired image, choose
%image forming method

if aspect<aspect0
    
    %Output image is 'taller' than I0. Need to pad in height, fit by width
    
    if width0 > width
        %If width0 is greater than 'width', interpolate image
        I = interp_image( I0 , width , height , 'width' ) ;
        
    elseif width0 < width
        
        %If width0 is less than 'width', montage several axes shifted and
        %zoomed images
        if strcmp( zoom_method,'interp')
            I = interp_image( I0 , width , height , 'width' );
        else
            I = zoom_montage( fig, I0 , width , height , 'width');
        end
    else
        %Width0 equates to 'width'
        I = I0;
    end
    
    %Pad with blank colours in height
    [heightI,widthI,num_cols]  = size(I);
    num_blanks = round( 0.5 * (height - heightI) );
    I = addblanks( I , num_blanks , 'height' , blank );
    
else
    
    %Output image is 'wider' than I0. Need to pad in width, fit by height
    
    if height0 > height
        
        %If height0 is greater than 'height', interpolate image
        I = interp_image( I0 , width , height , 'height' ) ;
        
    elseif height0 < height
        
        %If height0 is less than 'height', montage several axes shifted and
        %zoomed images.
        if strcmp( zoom_method,'interp')
            I = interp_image( I0 , width , height , 'height' );
        else
            I = zoom_montage( fig, I0 , width , height , 'height' );
        end
        
    else
        %height0 equates to 'height'
        I = I0;
    end
    
    %Pad with blank colours in width
    [heightI,widthI,num_cols]  = size(I);
    num_blanks = round( 0.5 * (width - widthI) );
    I = addblanks( I , num_blanks , 'width' , blank );
end

%Save final image
disp('.... Writing image to disk')
imwrite( uint8(I) , [name,'.',bitmap] , bitmap );

%Display ending message
disp(['.... Bitmap ',name,'.',bitmap,' saved in ',num2str(toc),' seconds.'])

%%

%Add blanks function
function II = addblanks( I , num_blanks , height_or_width , blank_RGB )

%Message to command window
disp('.... Adding blanks')

%Get dimensions of I
[heightI,widthI,num_cols]  = size(I);

%Create R,G,B matricies from image matrix I.
R = ones(heightI,widthI);
G = ones(heightI,widthI);
B = ones(heightI,widthI);
R(:,:) = I(:,:,1);
G(:,:) = I(:,:,2);
B(:,:) = I(:,:,3);

%Construct new RGB array II with required number of 'blanks' added
if strcmp(height_or_width,'width')
    II = ones( heightI,widthI+2*num_blanks,3);
    II(:,:,1) = [ blank_RGB(1)*ones(heightI,num_blanks) ,...
        R , blank_RGB(1)*ones(heightI,num_blanks) ];
    II(:,:,2) = [ blank_RGB(2)*ones(heightI,num_blanks),...
        G , blank_RGB(2)*ones(heightI,num_blanks) ];
    II(:,:,3) = [ blank_RGB(3)*ones(heightI,num_blanks) ,...
        B , blank_RGB(3)*ones(heightI,num_blanks) ];
else
    II = ones( heightI+2*num_blanks,widthI,3);
    II(:,:,1) = [ blank_RGB(1)*ones(num_blanks,widthI) ;...
        R ; blank_RGB(1)*ones(num_blanks,widthI) ];
    II(:,:,2) = [ blank_RGB(2)*ones(num_blanks,widthI) ;...
        G ; blank_RGB(2)*ones(num_blanks,widthI) ];
    II(:,:,3) = [ blank_RGB(3)*ones(num_blanks,widthI) ;...
        B ; blank_RGB(3)*ones(num_blanks,widthI) ];
end

%%

%Zoom and montage function. This function creates a matrix of images of
%figure windows. These stitched together will form the enlargement of the
%screen image. Enlargement is achieved by a linear scaling of the axes pf
%the figure and then a translation of the axes to correctly position the
%axes image such that the original contents of the figure comprise the
%final stitched image.
function [II,Z] = zoom_montage( fig, I , II_width , II_height , fit_by,...
    x_label,y_label,z_label,title_str)

%Get dimensions of I
[heightI,widthI,num_cols]  = size(I);

%Compute initial image aspect
aspectI = widthI/heightI;

%Define zoom factor and assign numbers of pixels of II in height and width
if strcmp(fit_by,'height')
    height = II_height;
    Z = height/heightI;
    width = round(Z*widthI);
else
    width = II_width;
    Z = width/widthI;
    height = round(Z*heightI);
end

%Initailise output image
II = [];

%Set renderer to be OpenGL. Zbuffer can cause problems at edges of figures.
figure(fig);
set(fig,'renderer','opengl');

%Obtain coordinates (relative to figure bottom left hand corner) of axes,
%in units of pixels.
orig_ax_units = get(gca,'units');
orig_fig_units = get(fig,'units');
set(fig,'units','pixels');
set(gca,'units','pixels');
p = get(gca,'Position');
orig_p = p;

%Zoom image by factor Z by modifying the position vector of the axes
%rectangle, relative to the figure.

%We need to shift such that the centre of the figure pre-zoom is now the centre of
%the figure post zoom. To achieve this find out the pixel displacements
%before and after zoom from the bottom left hand corner of the axis.
orig_c_pixels_x_from_bot_left = widthI/2-p(1);
orig_c_pixels_y_from_bot_left = heightI/2-p(2);
new_c_pixels_x_from_bot_left = Z*orig_c_pixels_x_from_bot_left;
new_c_pixels_y_from_bot_left = Z*orig_c_pixels_y_from_bot_left;
dx = orig_c_pixels_x_from_bot_left - new_c_pixels_x_from_bot_left;
dy = orig_c_pixels_y_from_bot_left - new_c_pixels_y_from_bot_left;

%Modify position vector p
p = [p(1)+dx,p(2)+dy,...
    Z*p(3),Z*p(4)];
set(gca,'position',p);

%Modify font size of axis tick marks
fsize = get(gca,'fontsize');
set(gca,'fontsize',Z*fsize);

%Modify font size of axis annotation. It is assumed axes labels have been
%defined to be the same font size as the axis tick marks
set(get(gca,'xlabel'),'fontsize',Z*fsize);
set(get(gca,'ylabel'),'fontsize',Z*fsize);
set(get(gca,'zlabel'),'fontsize',Z*fsize);
set(get(gca,'title'),'fontsize',Z*fsize);

%Set fontsize of any children of the axes
c = get(gca,'children');
if ~isempty(c)
    for n=1:length(c)
        try
            f = get(c(n),'fontsize');
            set( c(n),'fontsize',Z*f );
        end
    end
end

%Determine number of I sized images that fit into image sized width x
%height
num_w = ceil(width / widthI);
num_h = ceil(height / heightI);

%Compute sub-image centre pixel numbers (relative to width x height II
%image) and pixels of II image which correspond to sub-images. Determine
%sub-images by shifting the axes such that a matrix of figure windows
%covers the new (enlarged) image.
num = 0;
for nw = 1:num_w
    
    %Initialise 'row patches'
    II_rows = [];
    
    %Add rows
    for nh = 1: num_h
        
        %Increment sub-image number
        num = num + 1;
        
        %Display message to command window
        disp( ['Creating sub-image ',num2str(num),' of ',num2str(num_w*num_h)] );
        
        %Compute sub-image axes bottom left hand corner positions offsets
        wb =  -width/2 - widthI/2  + nw*widthI;
        hb =  -height/2 - heightI/2 + nh*heightI;
        
        %Define pixel numbers of image II that sub-image corresponds to
        indw = ((nw-1)*widthI + 1):(nw*widthI);
        indh = ((nh-1)*heightI + 1):(nh*heightI);
        
        %Re-position axes
        set(gca,'Position',[ p(1) - wb, p(2) - hb, p(3),p(4)] );
        
        %Make image
        F = getframe(fig);
        
        %Insert image into II
        F.cdata = F.cdata(1:length(indh),1:length(indw),:);
        II_rows = [F.cdata ;II_rows ];
    end
    
    %Add columns to II
    II = [II , II_rows];
end

%Trim off unwanted bits at the edges
dim = size(II);
II = II(dim(1)-height+1:dim(1),1:width,:);

%Unzoom to restore original figure
set(gca,'position',orig_p);
set(fig,'units',orig_fig_units);
set(gca,'units',orig_ax_units);

%Modify font size of axis tick marks
set(gca,'fontsize',fsize);

%Modify font size of axis annotation. It is assumed axes labels have been
%defined to be the same font size as the axis tick marks
set(get(gca,'xlabel'),'fontsize',fsize);
set(get(gca,'ylabel'),'fontsize',fsize);
set(get(gca,'zlabel'),'fontsize',fsize);
set(get(gca,'title'),'fontsize',fsize);

%%

%Test function - paste into command window
function fig2pic_test
peaks
axis vis3d
axis tight
fig=  gcf;
ax = gca;
%cropfig(fig,ax)
aspect = 6/4; DPI =300; fig2pic( gcf, 'test', 'png', aspect , DPI ,...
    aspect*10.2,'' );

%End of code

