%Hwavefunc
% Computes the wavefunction and probability density of Hydrogenic atoms.
%
% LAST UPDATED by Andy French 30th March 2011
%
% Syntax: [W,P,E] = Hwavefunc( n, L, M, Z, m_nuc,  r, azi, elev )
%
% n is the quantum number 0,1,2....
% L is the orbital quantum number 0,1,2...(n-1)
% M is the magnetic quantum number 0,+/-1, +/-2,...+/-L
% Z is the atomic number
% m_nuc is the atomic mass /kg
% r is the radius /angstrom
% azi is teh azimuth /radians
% elev is the elevation /radians
%
% W is the wavefunction amplitude (which may be complex)
% P is the probability density function = W.*conj(W)
% E is the total energy /eV of all the orbitals with quantum number n
%
% Note r, azi and elev can be matrices but they must be of the same size.

function [W,P,E] = Hwavefunc(  n, L, M, Z, m_nuc,  r, azi, elev )

%% Input parameters %%

e0 = 8.854e-12;  %Permittivity of free space / Fm^-1
h = 6.626e-34;   %Planck's constant / Js
me = 9.109e-31;  %Mass of the electron /kg
qe = 1.602e-19;  %Charge on the electron /coulombs

%

%Bohr radius / angstroms
a0 = e0*h*h/(pi*me*qe*qe);
a0 = a0/1e-10;

%Reduced mass
mu = me*m_nuc/(me+m_nuc);

%Compute wavefunction
a = me*a0/(mu*Z);
x = 2*r/(a*n);
A = sqrt( factorial(n-L-1)/( 2*n*factorial(n+L) ) );
B = (2/(a*n))^(3/2);
C=zeros(size(r));
for k=0 : n-L-1
    C = C + ( (-x).^k ) * factorial( L + n )/ ( factorial( 2*L + 1 + k ) * factorial( n-L-1-k ) * factorial(k) );
end
W = A * B * (x.^L) .* exp( -x/2 ).*C .* spharm( azi, elev, L, M ) ;
P = W.*conj(W);

%Orbital energy /eV
E = -mu*(qe^4)*(Z^2)/( 8*e0*e0*h*h*n*n );
E = E/qe;

%%

function Hwavefunc_demo

Z = 50;
rmax = 50;
m_nuc = 1.67e-27;
n=5;
L=4;
M=-4;

N=1000;
r = linspace( 0 , rmax, N );
azi = 0 * ones(1,N);
elev = 0 * ones(1,N);

[W,P,E] = Hwavefunc(  n, L, M, Z, m_nuc,  r, azi, elev );
figure('color',[1 1 1],'name','Hydrogenic wave functions','units','pixels','position',[519 327 909 527]);
plot(r,P);
xlabel('Radius /angstroms')
ylabel('Probability density')
title(['Hydrogenic atom probability density. Z=',num2str(Z),', m_{nuc}=',...
    num2str(m_nuc),' kg, n=',num2str(n),' L=',num2str(L),', M=',num2str(M),', E=',num2str(E),' eV' ] );

%End of code